from segment_anything import sam_model_registry, SamAutomaticMaskGenerator, SamPredictor
from draw_box_utils import draw_objs
from backbone import resnet50_fpn_backbone
from network_files import MaskRCNN
import cv2
from torchvision import transforms
import torch
import matplotlib.pyplot as plt
from PIL import Image
import numpy as np
import os
import time
import json
import shutil
os.environ["cuda_visible_devices"] = "0"


def create_model(num_classes, box_thresh=0.5):
    backbone = resnet50_fpn_backbone()
    model = MaskRCNN(backbone,
                     num_classes=num_classes,
                     rpn_score_thresh=box_thresh,
                     box_score_thresh=box_thresh)

    return model


def time_synchronized():
    torch.cuda.synchronize() if torch.cuda.is_available() else None
    return time.time()


def main(args):
    num_classes = 2  # 不包含背景
    box_thresh = 0.1
    file = args.image
    label_json_path = './coco91_indices_ins.json'
    filename,ext = os.path.splitext(os.path.basename(file))
    
    assert ext != 'jpg' and ext != 'JPG', '请输入jpg或JPG格式的图片'

    # get devices
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    print("using {} device.".format(device))

    # create model
    model = create_model(num_classes=num_classes + 1, box_thresh=box_thresh)

    # configure sam
    sam_checkpoint = args.seg_model
    model_type = "default"
    import sys
    sys.path.append("..")
    from segment_anything import sam_model_registry, SamAutomaticMaskGenerator, SamPredictor
    sam = sam_model_registry[model_type](
        checkpoint=sam_checkpoint)
    sam.to(device=device)
    mask_generator_2 = SamAutomaticMaskGenerator(
        model=sam,
        points_per_side=32,
        pred_iou_thresh=0.95,
        stability_score_thresh=0.92,
        crop_n_layers=1,
        crop_n_points_downscale_factor=2,
        min_mask_region_area=25,  # Requires open-cv to run post-processing
    )
    weights_path = args.model
    assert os.path.exists(
        weights_path), "{} file dose not exist.".format(weights_path)
    weights_dict = torch.load(weights_path, map_location='cpu')
    weights_dict = weights_dict["model"] if "model" in weights_dict else weights_dict
    model.load_state_dict(weights_dict)
    model.to(device)

    # read class_indict
    assert os.path.exists(
        label_json_path), "json file {} dose not exist.".format(label_json_path)
    with open(label_json_path, 'r') as json_file:
        category_index = json.load(json_file)


    assert os.path.exists(file), f"{file} does not exits."
    original_img = Image.open(file).convert('RGB')
    image = cv2.cvtColor(np.array(original_img), cv2.COLOR_RGB2BGR)
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    # from pil image to tensor, do not normalize image
    data_transform = transforms.Compose([transforms.ToTensor()])
    img = data_transform(original_img)
    # expand batch dimension
    img = torch.unsqueeze(img, dim=0)

    model.eval()  # 进入验证模式
    with torch.no_grad():
        # init
        img_height, img_width = img.shape[-2:]
        init_img = torch.zeros(
            (1, 3, img_height, img_width), device=device)
        model(init_img)

        predictions = model(img.to(device))[0]
        predict_boxes = predictions["boxes"].to("cpu").numpy()
        predict_classes = predictions["labels"].to("cpu").numpy()
        predict_scores = predictions["scores"].to("cpu").numpy()
        predict_mask = predictions["masks"].to("cpu").numpy()
        # [batch, 1, h, w] -> [batch, h, w]
        predict_mask = np.squeeze(predict_mask, axis=1)
        deleted = np.array([])
        for i in range(len(predict_classes)):
            if predict_classes[i] == 2:
                deleted = np.append(deleted, [i])
        deleted = deleted.astype(int)
        predict_boxes = np.delete(predict_boxes, deleted, 0)
        predict_classes = np.delete(predict_classes, deleted)
        predict_scores = np.delete(predict_scores, deleted)
        predict_mask = np.delete(predict_mask, deleted, 0)
        start_time = time.time()
        masks2 = mask_generator_2.generate(image)
        plot_img, teethMasks = draw_objs(original_img,
                                            boxes=predict_boxes,
                                            classes=predict_classes,
                                            scores=predict_scores,
                                            masks=predict_mask,
                                            masks2=masks2,
                                            category_index=category_index,
                                            line_thickness=8,
                                            font='arial.ttf',
                                            font_size=25)
        plot_img.save(os.path.join(args.save_dir,f'{filename}_pred{ext}'))
        print(f'{file}保存成功')

if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    
    parser.add_argument('--image',required=True,help='image to be predicted')
    parser.add_argument('--model',default='./save_weights/model_best.pth')
    parser.add_argument('--save_dir',default='./result')
    parser.add_argument('--seg_model',default='./seg_model/sam_vit_h_4b8939.pth')

    args = parser.parse_args()
    main(args)
